@@ -32,7 +32,11 @@ class SpecialEvent(TypedDict):
         "overlay",
         # Shortcuts
         "xbox_b",
+        "xbox_b_internal",
         "xbox_y",
+        "xbox_y_internal",
+        "xbox_a",
+        "xbox_x",
         "kbd_meta_press",
         "kbd_meta_hold",
         "swipe_left_top",
@@ -45,8 +49,21 @@ class SpecialEvent(TypedDict):
         "tdp_cycle_balanced",
         "tdp_cycle_performance",
         "tdp_cycle_custom",
+        "tdp_cycle",
         # Sleep information
         "wakeup",
+        # General Events
+        "brightness_changed",
+        "custom_armoury",
+        "custom_fan",
+        # Powerbutton presses
+        "pbtn_short",
+        "pbtn_long",
+        "pbtn_double",  # todo
+        # Debug
+        "restart_dev",
+        "shutdown_dev",
+        "refresh",
     ]
     data: "NotRequired[Any]"
 
@@ -158,6 +175,8 @@ class ConfigurationEvent(TypedDict):
 class RgbCapabilities(TypedDict):
     modes: dict[RgbMode, Sequence[RgbSettings]] | None
     controller: bool
+    resets_on_ac: bool
+    init_times: NotRequired[int | None]
     zones: RgbZones
 
 
@@ -236,8 +255,9 @@ class ControllerEmitter:
                     )
                     self.grab(False)
                     return False
-                elif evs and self._controller_cb:
-                    self._controller_cb(cid, evs)
+                elif self._controller_cb:
+                    if evs:
+                        self._controller_cb(cid, evs)
                     return True
                 else:
                     return False
@@ -317,6 +337,7 @@ TouchpadCorrectionType = Literal[
     "left",
     "right",
     "center",
+    "legos",
     "disabled",
 ]
 
@@ -346,7 +367,7 @@ def correct_touchpad(
                     y_mult=new_height,
                     y_ofs=(height - new_height),
                 )
-        case "right":
+        case "right" | "legos":
             if ratio > 2:
                 new_width = width / ratio
                 return TouchpadCorrection(
@@ -466,6 +487,8 @@ def correct_touchpad(
                 )
         case "stretch" | "disabled":
             return TouchpadCorrection(x_mult=width, y_mult=height)
+        # case "legos":
+        #     return TouchpadCorrection(x_mult=800, y_mult=800, x_ofs=1055, y_ofs=140)
 
     logger.error(f"Touchpad correction method '{method}' not found.")
     return TouchpadCorrection(x_mult=width, y_mult=height)
@@ -512,7 +535,7 @@ TouchpadAction = Literal["disabled", "left_click", "right_click"]
 
 
 class Multiplexer:
-    QAM_HOLD_TIME = 0.5
+    QAM_HOLD_TIME = 0.4
     QAM_MULTI_PRESS_DELAY = 0.2
     QAM_TAP_TIME = 0.04
     QAM_DELAY = 0.15
@@ -523,6 +546,7 @@ class Multiplexer:
     REBOOT_VIBRATION_OFF = 1.2
     REBOOT_VIBRATION_NUM = 3
     STEAM_CHECK_INTERVAL = 3
+    STARTSELECT_TRIGGER_THRESHOLD = 0.6
 
     def __init__(
         self,
@@ -533,6 +557,8 @@ class Multiplexer:
                 "guide_is_select",
                 "select_is_guide",
                 "start_is_keyboard",
+                "aya_traditional",
+                "aya_traditional_rev",
             ]
         ) = None,
         trigger: None | Literal["analog_to_discrete", "discrete_to_analogue"] = None,
@@ -548,7 +574,7 @@ class Multiplexer:
         ) = None,
         status: None | Literal["both_to_main"] = None,
         share_to_qam: bool = False,
-        trigger_discrete_lvl: float = 0.99,
+        trigger_discrete_lvl: float = 0.8,
         touchpad_short: TouchpadAction = "disabled",
         touchpad_right: TouchpadAction = "left_click",
         touchpad_hold: TouchpadAction = "disabled",
@@ -563,8 +589,10 @@ class Multiplexer:
         qam_multi_tap: bool = True,
         qam_no_release: bool = False,
         qam_hhd: bool = False,
+        qam_hold: Literal["hhd", "mode"] = "hhd",
         keyboard_is: Literal["steam_qam", "qam", "keyboard"] = "keyboard",
         keyboard_no_release: bool = False,
+        startselect_chord: str = "disabled",
     ) -> None:
         self.swap_guide = swap_guide
         self.trigger = trigger
@@ -589,8 +617,11 @@ class Multiplexer:
         self.send_xbox_b = None
         self.imu = imu
         self.qam_hhd = qam_hhd
+        self.qam_hold = qam_hold
         self.keyboard_is = keyboard_is
         self.keyboard_no_release = keyboard_no_release
+        self.startselect_chord = startselect_chord
+        self.startselect_pressed = None
 
         self.state = {}
         self.touchpad_x = 0
@@ -638,6 +669,8 @@ class Multiplexer:
                     "modes": rgb_modes,
                     "controller": uses_rgb,
                     "zones": rgb_zones,
+                    "resets_on_ac": params.get("rgb_resets_on_ac", False),
+                    "init_times": params.get("rgb_init_times", None),
                 }
             self.emit.set_capabilities(
                 self.unique,
@@ -663,10 +696,10 @@ class Multiplexer:
             if ev == "reboot":
                 if self.reboot_is_held:
                     try:
-                        import os
+                        import subprocess
 
-                        os.system("systemctl reboot")
-                        logger.info("rebooting")
+                        subprocess.run(["reboot"])
+                        logger.info("Rebooting")
                     except Exception as e:
                         logger.error(f"Rebooting failed with error:\n{type(e)}:{e}")
             elif self.reboot_is_held or not ev.get("from_reboot", False):
@@ -721,9 +754,8 @@ class Multiplexer:
         if (
             self.touchpad_hold != "disabled"
             and self.touchpad_down
+            and self.touchpad_down[3]
             and curr - self.touchpad_down[0] > 0.8
-            and abs(self.touchpad_down[1] - self.touchpad_x) < 0.1
-            and abs(self.touchpad_down[2] - self.touchpad_y) < 0.1
         ):
             action = (
                 "touchpad_left"
@@ -751,6 +783,11 @@ class Multiplexer:
                 )
             )
             self.touchpad_down = None
+        elif self.touchpad_down and (
+            abs(self.touchpad_down[1] - self.touchpad_x) > 0.13
+            or abs(self.touchpad_down[2] - self.touchpad_y) > 0.13
+        ):
+            self.touchpad_down[3] = False
 
         for ev in events:
             match ev["type"]:
@@ -812,6 +849,38 @@ class Multiplexer:
                                     ev["code"] = "right_imu_ts"
                                     ev["code"] = "left_imu_ts"
 
+                    if (
+                        self.startselect_pressed == "wait"
+                        and ev["code"]
+                        in (
+                            "lt",
+                            "rt",
+                            "hat_x",
+                            "hat_y",
+                        )
+                        and abs(ev["value"]) > self.STARTSELECT_TRIGGER_THRESHOLD
+                    ):
+                        out.append(
+                            {
+                                "type": "button",
+                                "code": "mode",
+                                "value": True,
+                            }
+                        )
+                        self.startselect_pressed = "pressed"
+                    if self.startselect_pressed == "pressed":
+                        self.queue.append(
+                            (
+                                {
+                                    "type": "axis",
+                                    "code": ev["code"],
+                                    "value": ev["value"],
+                                },
+                                curr + self.QAM_DELAY,
+                            )
+                        )
+                        ev["code"] = ""  # type: ignore
+
                     if self.trigger == "analog_to_discrete" and ev["code"] in (
                         "lt",
                         "rt",
@@ -893,6 +962,10 @@ class Multiplexer:
                             # the arguments do not make sense.
                             case "start":
                                 match self.swap_guide:
+                                    case "aya_traditional":
+                                        ev["code"] = "select"
+                                    case "aya_traditional_rev":
+                                        ev["code"] = "mode"
                                     case "start_is_keyboard":
                                         ev["code"] = "keyboard"
                                     case "select_is_guide":
@@ -901,6 +974,10 @@ class Multiplexer:
                                         ev["code"] = "mode"
                             case "select":
                                 match self.swap_guide:
+                                    case "aya_traditional":
+                                        ev["code"] = "mode"
+                                    case "aya_traditional_rev":
+                                        pass
                                     case "start_is_keyboard":
                                         ev["code"] = "mode"
                                     case "select_is_guide":
@@ -908,12 +985,21 @@ class Multiplexer:
                                     case _:
                                         ev["code"] = "share"
                             case "mode":
-                                if self.swap_guide == "guide_is_start":
-                                    ev["code"] = "start"
-                                else:
-                                    ev["code"] = "select"
+                                match self.swap_guide:
+                                    case "aya_traditional":
+                                        ev["code"] = "start"
+                                    case "aya_traditional_rev":
+                                        ev["code"] = "keyboard"
+                                    case "guide_is_start":
+                                        ev["code"] = "start"
+                                    case _:
+                                        ev["code"] = "select"
                             case "share":
                                 match self.swap_guide:
+                                    case "aya_traditional":
+                                        pass
+                                    case "aya_traditional_rev":
+                                        pass
                                     case "start_is_keyboard":
                                         pass
                                     case "guide_is_start":
@@ -921,8 +1007,53 @@ class Multiplexer:
                                     case _:
                                         ev["code"] = "start"
                             case "keyboard":
-                                if self.swap_guide == "start_is_keyboard":
-                                    ev["code"] = "start"
+                                match self.swap_guide:
+                                    case "start_is_keyboard":
+                                        ev["code"] = "start"
+                                    case "aya_traditional_rev":
+                                        ev["code"] = "start"
+
+                    if (
+                        self.startselect_chord != "disabled" and ev["code"] == "select"
+                    ) or (
+                        self.startselect_chord == "start_select"
+                        and ev["code"] == "start"
+                    ):
+                        if self.startselect_pressed == "pressed":
+                            self.queue.append(
+                                (
+                                    {
+                                        "type": "button",
+                                        "code": "mode",
+                                        "value": False,
+                                    },
+                                    curr + self.QAM_DELAY,
+                                )
+                            )
+                            self.startselect_pressed = None
+
+                        if ev["value"]:
+                            self.startselect_pressed = "wait"
+                        elif self.startselect_pressed == "wait":
+                            self.startselect_pressed = None
+                            out.append(
+                                {
+                                    "type": "button",
+                                    "code": ev["code"],
+                                    "value": True,
+                                }
+                            )
+                            self.queue.append(
+                                (
+                                    {
+                                        "type": "button",
+                                        "code": ev["code"],
+                                        "value": False,
+                                    },
+                                    curr + self.QAM_DELAY,
+                                )
+                            )
+                        ev["code"] = ""  # type: ignore
 
                     if self.emit and ev["code"] == "mode":
                         # Steam might do weirdness, emit an event to prepare
@@ -1174,7 +1305,7 @@ class Multiplexer:
                         and ev["code"] == "y"
                         and ev["value"]
                     ):
-                        self.emit({"type": "special", "event": "xbox_y"})
+                        self.emit({"type": "special", "event": "xbox_y_internal"})
 
                     # Assume we can only use Xbox + B for short presses
                     if (
@@ -1189,9 +1320,31 @@ class Multiplexer:
                                 self.send_xbox_b
                                 and time.time() - self.send_xbox_b < 0.3
                             ):
-                                self.emit({"type": "special", "event": "xbox_b"})
+                                self.emit({"type": "special", "event": "xbox_b_internal"})
                             self.send_xbox_b = None
 
+                    # Apply start/select qam
+                    if self.startselect_pressed == "wait" and ev["code"]:
+                        out.append(
+                            {
+                                "type": "button",
+                                "code": "mode",
+                                "value": True,
+                            }
+                        )
+                        self.startselect_pressed = "pressed"
+                    if self.startselect_pressed == "pressed":
+                        self.queue.append(
+                            (
+                                {
+                                    "type": "button",
+                                    "code": ev["code"],
+                                    "value": ev["value"],
+                                },
+                                curr + self.QAM_DELAY,
+                            )
+                        )
+                        ev["code"] = ""  # type: ignore
                 case "led":
                     if self.led == "left_to_main" and ev["code"] == "left":
                         out.append({**ev, "code": "main"})
@@ -1212,11 +1365,12 @@ class Multiplexer:
                                 status_events.add("is_connected")
 
         if touched:
-            self.touchpad_down = (
+            self.touchpad_down = [
                 curr,
                 self.touchpad_x,
                 self.touchpad_y,
-            )
+                bool(True),
+            ]
 
         for s in status_events:
             match s:
@@ -1290,6 +1444,9 @@ class Multiplexer:
         send_steam_qam = send_steam_qam or (
             qam_apply and not qam_hhd and self.qam_released and self.qam_times == 1
         )
+        send_steam_expand = (
+            qam_apply and self.qam_pressed and was_held and self.qam_hold == "mode"
+        )
         if qam_apply and self.emit:
             if qam_hhd:
                 match self.qam_times:
@@ -1300,7 +1457,9 @@ class Multiplexer:
                     case _:
                         self.emit({"type": "special", "event": "qam_triple"})
             else:
-                if self.qam_pressed and was_held:
+                # FIXME: hiding the event based on qam_hold should not happen
+                # instead the handler should not open hhd
+                if self.qam_pressed and was_held and self.qam_hold == "hhd":
                     self.emit({"type": "special", "event": "qam_hold"})
                 else:
                     match self.qam_times:
@@ -1337,16 +1496,26 @@ class Multiplexer:
         if self.emit and self.emit.intercept(
             self.unique, [o for o in out if not o.get("from_queue", False)]
         ):
-            accel = random.random() * 10
-            fake_accel: Sequence[Event] = [
-                {"type": "axis", "code": "accel_x", "value": accel},
-                {"type": "axis", "code": "left_accel_x", "value": accel},
-                {"type": "axis", "code": "right_accel_x", "value": accel},
-            ]
-            return fake_accel + [
+            # Hiding the gyro and sending fake accel values causes
+            # issues with emulators and gyro
+            # accel = random.random() * 10
+            # fake_accel: Sequence[Event] = [
+            #     {"type": "axis", "code": "accel_x", "value": accel},
+            #     {"type": "axis", "code": "left_accel_x", "value": accel},
+            #     {"type": "axis", "code": "right_accel_x", "value": accel},
+            # ]
+            # return fake_accel + [
+            #     o
+            #     for o in out
+            #     if o["type"] not in ("button", "axis") or "ts" in o.get("code", "")
+            # ]
+            return [
                 o
                 for o in out
-                if o["type"] not in ("button", "axis") or "ts" in o.get("code", "")
+                if o["type"] not in ("button", "axis")
+                or "ts" in o.get("code", "")
+                or "accel" in o.get("code", "")
+                or "gyro" in o.get("code", "")
             ]
         elif send_steam_qam:
             # Send steam qam only if not intercepting
