@@ -156,13 +156,15 @@ def is_device(fn):
     if not os.path.exists(fn):
         return False
 
-    m = os.stat(fn)[stat.ST_MODE]
-    if not stat.S_ISCHR(m):
-        return False
+    try:
+        m = os.stat(fn)[stat.ST_MODE]
+        if not stat.S_ISCHR(m):
+            return False
 
-    if not os.access(fn, os.R_OK | os.W_OK):
+        if not os.access(fn, os.R_OK | os.W_OK):
+            return False
+    except Exception:
         return False
-
     return True
 
 
@@ -179,7 +181,7 @@ def list_evs(filter_valid: bool = False, fn: str = "/proc/bus/input/devices"):
                 continue
             match line[0]:
                 case "I":
-                    for attr in line[3:-1].split(" "):
+                    for attr in line[3:].split(" "):
                         name, val = attr.split("=")
                         out[name.lower()] = int(val, 16)
                 case "N":
@@ -242,6 +244,7 @@ class GenericGamepadEvdev(Producer, Consumer):
         msc_map: Mapping[int, Button] = {},
         msc_delay: float = 0.1,
         postprocess: dict[str, dict] = AXIS_CALIBRATION,
+        requires_start: bool = False,
     ) -> None:
         self.vid = vid
         self.pid = pid
@@ -263,6 +266,8 @@ class GenericGamepadEvdev(Producer, Consumer):
         self.queue = []
         self.postprocess = postprocess
         self.start_pressed = None
+        self.start_held = False
+        self.requires_start = requires_start
 
     def open(self) -> Sequence[int]:
         for d, info in list_evs(filter_valid=True).items():
@@ -280,8 +285,12 @@ class GenericGamepadEvdev(Producer, Consumer):
                     if cap_id not in dev_cap:
                         matches = False
                         break
+                    if cap_id != B("EV_ABS"):
+                        dev_caps = dev_cap[cap_id]
+                    else:
+                        dev_caps = [c[0] for c in dev_cap[cap_id]]  # type: ignore
                     for cap in caps:
-                        if cap not in dev_cap[cap_id]:
+                        if cap not in dev_caps:
                             matches = False
                         break
                 if not matches:
@@ -420,7 +429,11 @@ class GenericGamepadEvdev(Producer, Consumer):
         while can_read(self.fd):
             for e in self.dev.read():
                 if e.type == B("EV_KEY"):
-                    if e.code in self.btn_map:
+                    if e.code == B("KEY_LEFTMETA"):
+                        self.start_held = e.value != 0
+                    if e.code in self.btn_map and (
+                        not self.requires_start or self.start_held or not e.value
+                    ):
                         # Only 1 is valid for press (look at sysrq)
                         if e.code == B("KEY_LEFTMETA") and e.value:
                             # start requires special handling
