@@ -2,7 +2,10 @@ import logging
 import time
 from typing import Sequence
 
-from evdev import UInput
+try:
+    from evdev.uinput import UInput
+except ImportError:
+    from evdev import UInput # type: ignore[no-redef]
 
 from hhd.controller.base import Consumer, Event, Producer, can_read
 from hhd.controller.const import Axis, Button
@@ -17,7 +20,7 @@ _cache = ControllerCache()
 _cache_motions = ControllerCache()
 _cache_volume = ControllerCache()
 
-MIN_TIME_FOR_CACHE = 2
+MAX_IMU_SYNC_DELAY = 2
 
 
 class UInputDevice(Consumer, Producer):
@@ -49,6 +52,8 @@ class UInputDevice(Consumer, Producer):
         cache: bool = False,
         motions_device: bool = False,
         volume_keyboard: bool = False,
+        sync_gyro: bool = False,
+        gyro: bool = True,
     ) -> None:
         self.capabilities = capabilities
         self.btn_map = btn_map
@@ -70,6 +75,11 @@ class UInputDevice(Consumer, Producer):
         self.cache = cache
         self.motions_device = motions_device
         self.volume_keyboard = volume_keyboard
+        self.sync_gyro = sync_gyro
+        self.imu_failed = False
+        self.last_imu = 0
+        self.wrote = False
+        self.gyro = gyro
         if volume_keyboard:
             self.cache = True
 
@@ -98,16 +108,16 @@ class UInputDevice(Consumer, Producer):
                     and self.vid == cached.vid
                     and self.pid == cached.pid
                     and self.bus == cached.bus
+                    and self.version == cached.version
                     and self.phys == cached.phys
                     and self.input_props == cached.input_props
                     and self.uniq == cached.uniq
+                    and self.gyro == cached.gyro
                 ):
-                    logger.warning(
-                        f"Using cached controller node for {name}."
-                    )
+                    logger.warning(f"Using cached controller node for {name}.")
                     self.dev = cached.dev
                 else:
-                    cached.close(True)
+                    cached.close(True, in_cache=True)
 
         if not self.dev:
             try:
@@ -141,6 +151,9 @@ class UInputDevice(Consumer, Producer):
         self.touch_id = 1
         self.fd = self.dev.fd
         self.start = time.perf_counter()
+        self.last_imu = time.perf_counter()
+        self.imu_failed = False
+        self.wrote = False
 
         if self.ignore_cmds:
             # Do not wake up if we ignore to save utilization
@@ -160,9 +173,7 @@ class UInputDevice(Consumer, Producer):
             else:
                 name = "controller"
                 _cache.add(self)
-            logger.warning(
-                f"Caching {name} to avoid reconnection."
-            )
+            logger.warning(f"Caching {name} to avoid reconnection.")
         elif self.dev:
             self.dev.close()
             self.dev = None
@@ -174,6 +185,7 @@ class UInputDevice(Consumer, Producer):
         if not self.dev:
             return
 
+        should_syn = not self.sync_gyro
         wrote = {}
         ts = 0
         for ev in reversed(events):
@@ -184,6 +196,11 @@ class UInputDevice(Consumer, Producer):
                 continue
             match ev["type"]:
                 case "axis":
+                    if not should_syn and "imu_ts" in ev["code"]:
+                        self.imu_failed = False
+                        self.last_imu = time.perf_counter()
+                        should_syn = True
+
                     if ev["code"] in self.axis_map:
                         ax = self.axis_map[ev["code"]]
                         if ev["code"] == "touchpad_x":
@@ -251,8 +268,22 @@ class UInputDevice(Consumer, Producer):
             ts = (time.perf_counter_ns() // 1000) % (2**31)
             self.dev.write(B("EV_MSC"), B("MSC_TIMESTAMP"), ts)
 
-        if wrote and (not self.output_imu_timestamps or ts):
+        if self.sync_gyro:
+            curr = time.perf_counter()
+            if curr - self.last_imu > MAX_IMU_SYNC_DELAY and not self.imu_failed:
+                self.imu_failed = True
+                logger.error(
+                    f"IMU Did not send information for {MAX_IMU_SYNC_DELAY}s. Disabling Gyro Sync."
+                )
+
+        self.wrote = self.wrote or bool(wrote)
+        if (
+            self.wrote
+            and (should_syn or not self.sync_gyro or self.imu_failed)
+            and (not self.output_imu_timestamps or ts)
+        ):
             self.dev.syn()
+            self.wrote = False
 
     def produce(self, fds: Sequence[int]) -> Sequence[Event]:
         if self.ignore_cmds or not self.fd or not self.fd in fds or not self.dev:
@@ -289,7 +320,7 @@ class UInputDevice(Consumer, Producer):
                     if self.rumble:
                         out.append(self.rumble)
                     else:
-                        logger.warn(
+                        logger.warning(
                             f"Rumble requested but a rumble effect has not been uploaded.\n{ev}"
                         )
                 elif ev.type == B("EV_FF") and not ev.value:
